# /* Copyright 2019 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================*/
#
# #include <memory>
# #include <string>
# #include <vector>
#
# #include "pybind11/pybind11.h"
# #include "pybind11/stl.h"
from ..... import core
# #include "tensorflow/core/lib/core/error_codes.pb.h"
# #include "tensorflow/core/lib/core/errors.h"
from ..... import errors
# #include "tensorflow/core/lib/core/status.h"
# #include "tensorflow/core/lib/io/buffered_inputstream.h"
from ....core.lib.io import buffered_inputstream
# #include "tensorflow/core/lib/io/random_inputstream.h"
from ....core.lib.io import random_inputstream
# #include "tensorflow/core/platform/env.h"
from ..... import env
# #include "tensorflow/core/platform/file_statistics.h"
# #include "tensorflow/core/platform/file_system.h"
from ..... import file_system
# #include "tensorflow/core/platform/stringpiece.h"
# #include "tensorflow/core/platform/tstring.h"
# #include "tensorflow/python/lib/core/pybind11_absl.h"
# #include "tensorflow/python/lib/core/pybind11_status.h"

from ...lib.core import py_exception_registry

def MaybeRaiseRegisteredFromStatusWithGIL(status: errors.Status):
  # TF_Code code = TF_GetCode(status);
  # if (code != TF_OK) {
  #   PyErr_SetObject(PyExceptionRegistry::Lookup(code),
  #                   pybind11::make_tuple(pybind11::none(), pybind11::none(),
  #                                        TF_Message(status))
  #                       .ptr());
  #   throw pybind11::error_already_set();
  # }
  if not status.ok():
    code = status.code()
    cls = py_exception_registry.PyExceptionRegistry.Lookup(code)
    exn = cls(None, None, status.error_message())
    raise exn

#
# namespace tensorflow {
# struct PyTransactionToken {
#   TransactionToken* token_;
# };
#
# inline TransactionToken* TokenFromPyToken(PyTransactionToken* t) {
#   return (t ? t->token_ : nullptr);
# }
# }  // namespace tensorflow
#
# namespace {
# namespace py = pybind11;
#
# PYBIND11_MODULE(_pywrap_file_io, m) {
#   using tensorflow::PyTransactionToken;
#   using tensorflow::TransactionToken;
#   py::class_<PyTransactionToken>(m, "TransactionToken")
#       .def("__repr__", [](const PyTransactionToken* t) {
#         if (t->token_) {
#           return std::string(t->token_->owner->DecodeTransaction(t->token_));
#         }
#         return std::string("Invalid token!");
#       });
#
#   m.def(
#       "FileExists",
#       [](const std::string& filename, PyTransactionToken* token) {
#         tensorflow::Status status;
#         {
#           py::gil_scoped_release release;
#           status = tensorflow::Env::Default()->FileExists(filename);
#         }
#         tensorflow::MaybeRaiseRegisteredFromStatusWithGIL(status);
#       },
#       py::arg("filename"), py::arg("token") = (PyTransactionToken*)nullptr);
def FileExists(filename: str):
  status = env.Env.default().file_exists(filename)
  MaybeRaiseRegisteredFromStatusWithGIL(status)

#   m.def(
#       "DeleteFile",
#       [](const std::string& filename, PyTransactionToken* token) {
#         py::gil_scoped_release release;
#         tensorflow::Status status =
#             tensorflow::Env::Default()->DeleteFile(filename);
#         tensorflow::MaybeRaiseRegisteredFromStatusWithGIL(status);
#       },
#       py::arg("filename"), py::arg("token") = (PyTransactionToken*)nullptr);
def DeleteFile(filename: str):
  status = env.Env.default().delete_file(filename)
  MaybeRaiseRegisteredFromStatusWithGIL(status)
#   m.def(
#       "ReadFileToString",
#       [](const std::string& filename, PyTransactionToken* token) {
#         std::string data;
#         py::gil_scoped_release release;
#         const auto status =
#             ReadFileToString(tensorflow::Env::Default(), filename, &data);
#         pybind11::gil_scoped_acquire acquire;
#         tensorflow::MaybeRaiseRegisteredFromStatus(status);
#         return py::bytes(data);
#       },
#       py::arg("filename"), py::arg("token") = (PyTransactionToken*)nullptr);
#   m.def(
#       "WriteStringToFile",
#       [](const std::string& filename, tensorflow::StringPiece data,
#          PyTransactionToken* token) {
#         py::gil_scoped_release release;
#         const auto status =
#             WriteStringToFile(tensorflow::Env::Default(), filename, data);
#         tensorflow::MaybeRaiseRegisteredFromStatusWithGIL(status);
#       },
#       py::arg("filename"), py::arg("data"),
#       py::arg("token") = (PyTransactionToken*)nullptr);
#   m.def(
#       "GetChildren",
#       [](const std::string& dirname, PyTransactionToken* token) {
#         std::vector<std::string> results;
#         py::gil_scoped_release release;
#         const auto status =
#             tensorflow::Env::Default()->GetChildren(dirname, &results);
#         pybind11::gil_scoped_acquire acquire;
#         tensorflow::MaybeRaiseRegisteredFromStatus(status);
#         return results;
#       },
#       py::arg("dirname"), py::arg("token") = (PyTransactionToken*)nullptr);
#   m.def(
#       "GetMatchingFiles",
#       [](const std::string& pattern, PyTransactionToken* token) {
#         std::vector<std::string> results;
#         py::gil_scoped_release release;
#         const auto status =
#             tensorflow::Env::Default()->GetMatchingPaths(pattern, &results);
#         pybind11::gil_scoped_acquire acquire;
#         tensorflow::MaybeRaiseRegisteredFromStatus(status);
#         return results;
#       },
#       py::arg("pattern"), py::arg("token") = (PyTransactionToken*)nullptr);
def GetMatchingFiles(pattern):
  status, results = env.Env.default().get_matching_paths(pattern)
  MaybeRaiseRegisteredFromStatusWithGIL(status)
  return results
#   m.def(
#       "CreateDir",
#       [](const std::string& dirname, PyTransactionToken* token) {
#         py::gil_scoped_release release;
#         const auto status = tensorflow::Env::Default()->CreateDir(dirname);
#         if (tensorflow::errors::IsAlreadyExists(status)) {
#           return;
#         }
#         tensorflow::MaybeRaiseRegisteredFromStatusWithGIL(status);
#       },
#       py::arg("dirname"), py::arg("token") = (PyTransactionToken*)nullptr);
#   m.def(
#       "RecursivelyCreateDir",
#       [](const std::string& dirname, PyTransactionToken* token) {
#         py::gil_scoped_release release;
#         const auto status =
#             tensorflow::Env::Default()->RecursivelyCreateDir(dirname);
#         tensorflow::MaybeRaiseRegisteredFromStatusWithGIL(status);
#       },
#       py::arg("dirname"), py::arg("token") = (PyTransactionToken*)nullptr);
#   m.def(
#       "CopyFile",
#       [](const std::string& src, const std::string& target, bool overwrite,
#          PyTransactionToken* token) {
#         py::gil_scoped_release release;
#         auto* env = tensorflow::Env::Default();
#         tensorflow::Status status;
#         if (!overwrite && env->FileExists(target).ok()) {
#           status = tensorflow::errors::AlreadyExists("file already exists");
#         } else {
#           status = env->CopyFile(src, target);
#         }
#         tensorflow::MaybeRaiseRegisteredFromStatusWithGIL(status);
#       },
#       py::arg("src"), py::arg("target"), py::arg("overwrite"),
#       py::arg("token") = (PyTransactionToken*)nullptr);
#   m.def(
#       "RenameFile",
#       [](const std::string& src, const std::string& target, bool overwrite,
#          PyTransactionToken* token) {
#         py::gil_scoped_release release;
#         auto* env = tensorflow::Env::Default();
#         tensorflow::Status status;
#         if (!overwrite && env->FileExists(target).ok()) {
#           status = tensorflow::errors::AlreadyExists("file already exists");
#         } else {
#           status = env->RenameFile(src, target);
#         }
#         tensorflow::MaybeRaiseRegisteredFromStatusWithGIL(status);
#       },
#       py::arg("src"), py::arg("target"), py::arg("overwrite"),
#       py::arg("token") = (PyTransactionToken*)nullptr);
def RenameFile(src, target, overwrite: bool):
  env_ = env.Env.default()
  if not overwrite and env_.file_exists(target).ok():
    status = errors.AlreadyExists("file already exists")
  else:
    status = env_.rename_file(src, target)
  MaybeRaiseRegisteredFromStatusWithGIL(status)

#   m.def(
#       "DeleteRecursively",
#       [](const std::string& dirname, PyTransactionToken* token) {
#         py::gil_scoped_release release;
#         tensorflow::int64 undeleted_files;
#         tensorflow::int64 undeleted_dirs;
#         auto status = tensorflow::Env::Default()->DeleteRecursively(
#             dirname, &undeleted_files, &undeleted_dirs);
#         if (status.ok() && (undeleted_files > 0 || undeleted_dirs > 0)) {
#           status = tensorflow::errors::PermissionDenied(
#               "could not fully delete dir");
#         }
#         tensorflow::MaybeRaiseRegisteredFromStatusWithGIL(status);
#       },
#       py::arg("dirname"), py::arg("token") = (PyTransactionToken*)nullptr);

#   m.def(
#       "IsDirectory",
#       [](const std::string& dirname, PyTransactionToken* token) {
#         py::gil_scoped_release release;
#         const auto status = tensorflow::Env::Default()->IsDirectory(dirname);
#         // FAILED_PRECONDITION response means path exists but isn't a dir.
#         if (tensorflow::errors::IsFailedPrecondition(status)) {
#           return false;
#         }
#
#         tensorflow::MaybeRaiseRegisteredFromStatusWithGIL(status);
#         return true;
#       },
#       py::arg("dirname"), py::arg("token") = (PyTransactionToken*)nullptr);
def IsDirectory(dirname):
  status = env.Env.default().is_directory(dirname)
  # FAILED_PRECONDITION response means path exists but isn't a dir.
  if errors.is_failed_precondition(status):
    return False
  MaybeRaiseRegisteredFromStatusWithGIL(status)
  return True

#   m.def("HasAtomicMove", [](const std::string& path) {
#     py::gil_scoped_release release;
#     bool has_atomic_move;
#     const auto status =
#         tensorflow::Env::Default()->HasAtomicMove(path, &has_atomic_move);
#     tensorflow::MaybeRaiseRegisteredFromStatusWithGIL(status);
#     return has_atomic_move;
#   });
def HasAtomicMove(path):
  status, has_atomic_move = env.Env.default().has_atomic_move(path)
  MaybeRaiseRegisteredFromStatusWithGIL(status)
  return has_atomic_move

#   py::class_<tensorflow::FileStatistics>(m, "FileStatistics")
#       .def_readonly("length", &tensorflow::FileStatistics::length)
#       .def_readonly("mtime_nsec", &tensorflow::FileStatistics::mtime_nsec)
#       .def_readonly("is_directory", &tensorflow::FileStatistics::is_directory);
#
#   m.def(
#       "Stat",
#       [](const std::string& filename, PyTransactionToken* token) {
#         py::gil_scoped_release release;
#         std::unique_ptr<tensorflow::FileStatistics> self(
#             new tensorflow::FileStatistics);
#         const auto status =
#             tensorflow::Env::Default()->Stat(filename, self.get());
#         py::gil_scoped_acquire acquire;
#         tensorflow::MaybeRaiseRegisteredFromStatus(status);
#         return self.release();
#       },
#       py::arg("filename"), py::arg("token") = (PyTransactionToken*)nullptr);
def Stat(filename):
  status, self = env.Env.default().stat(filename)
  MaybeRaiseRegisteredFromStatusWithGIL(status)
  return self

#   using tensorflow::WritableFile;
#   py::class_<WritableFile>(m, "WritableFile")
class WritableFile:
  # .def(py::init([](const std::string& filename, const std::string& mode,
  #                  PyTransactionToken* token) {
  #        py::gil_scoped_release release;
  #        auto* env = tensorflow::Env::Default();
  #        std::unique_ptr<WritableFile> self;
  #        const auto status = mode.find('a') == std::string::npos
  #                                ? env->NewWritableFile(filename, &self)
  #                                : env->NewAppendableFile(filename, &self);
  #        py::gil_scoped_acquire acquire;
  #        tensorflow::MaybeRaiseRegisteredFromStatus(status);
  #        return self.release();
  #      }),
  #      py::arg("filename"), py::arg("mode"),
  #      py::arg("token") = (PyTransactionToken*)nullptr)
  def __init__(self, filename, mode=b"w"):
    env_ = env.Env.default()
    status, me = env_.new_writable_file(filename) if b'a' not in mode else env.new_appendable_file(filename)
    MaybeRaiseRegisteredFromStatusWithGIL(status)
    self.me_ = me

  # .def("append",
  #      [](WritableFile* self, tensorflow::StringPiece data) {
  #        const auto status = self->Append(data);
  #        tensorflow::MaybeRaiseRegisteredFromStatusWithGIL(status);
  #      })
  def append(self, data):
    status = self.me_.append(core.string_view(data))
    MaybeRaiseRegisteredFromStatusWithGIL(status)

  # // TODO(slebedev): Make WritableFile::Tell const and change self
  # // to be a reference.
  # .def("tell",
  #      [](WritableFile* self) {
  #        tensorflow::int64 pos = -1;
  #        py::gil_scoped_release release;
  #        const auto status = self->Tell(&pos);
  #        tensorflow::MaybeRaiseRegisteredFromStatusWithGIL(status);
  #        return pos;
  #      })
  def tell(self):
    status, pos = self.me_.tell()
    MaybeRaiseRegisteredFromStatusWithGIL(status)
    return pos

  # .def("flush",
  #      [](WritableFile* self) {
  #        py::gil_scoped_release release;
  #        tensorflow::MaybeRaiseRegisteredFromStatusWithGIL(self->Flush());
  #      })
  def flush(self):
    MaybeRaiseRegisteredFromStatusWithGIL(self.me_.flush())

  # .def("close", [](WritableFile* self) {
  #   py::gil_scoped_release release;
  #   tensorflow::MaybeRaiseRegisteredFromStatusWithGIL(self->Close());
  # });
  def close(self):
    MaybeRaiseRegisteredFromStatusWithGIL(self.me_.close())
#
#   using tensorflow::io::BufferedInputStream;
#   py::class_<BufferedInputStream>(m, "BufferedInputStream")
class BufferedInputStream:
  # .def(py::init([](const std::string& filename, size_t buffer_size,
  #                  PyTransactionToken* token) {
  def __init__(self, filename, buffer_size: int):
    # py::gil_scoped_release release;
    # std::unique_ptr<tensorflow::RandomAccessFile> file;
    # const auto status =
    #     tensorflow::Env::Default()->NewRandomAccessFile(filename,
    #                                                     &file);
    status, file = env.Env.default().new_random_access_file(filename)
    # tensorflow::MaybeRaiseRegisteredFromStatusWithGIL(status);
    MaybeRaiseRegisteredFromStatusWithGIL(status)
    # std::unique_ptr<tensorflow::io::RandomAccessInputStream>
    #     input_stream(new tensorflow::io::RandomAccessInputStream(
    #         file.release(),
    #         /*owns_file=*/true));
    input_stream = random_inputstream.RandomAccessInputStream(file, owns_file=True)
    # py::gil_scoped_acquire acquire;
    # return new BufferedInputStream(input_stream.release(), buffer_size,
    #                                /*owns_input_stream=*/true);
    self.me_ = buffered_inputstream.BufferedInputStream(input_stream, buffer_size, owns_input_stream=True)
  #      }),
  #      py::arg("filename"), py::arg("buffer_size"),
  #      py::arg("token") = (PyTransactionToken*)nullptr)

  #     .def("read",
  #          [](BufferedInputStream* self, tensorflow::int64 bytes_to_read) {
  def read(self, bytes_to_read: int):
    # py::gil_scoped_release release;
    # tensorflow::tstring result;
    # const auto status = self->ReadNBytes(bytes_to_read, &result);
    status, result = self.me_.read_n_bytes(bytes_to_read)
    # if (!status.ok() && !tensorflow::errors::IsOutOfRange(status)) {
    #   result.clear();
    #   tensorflow::MaybeRaiseRegisteredFromStatusWithGIL(status);
    # }
    if not status.ok() and not errors.is_out_of_range(status):
      result.clear()
      MaybeRaiseRegisteredFromStatusWithGIL(status)
    # py::gil_scoped_acquire acquire;
    # return py::bytes(result);
    return bytes(result)
  #          })

  #     .def("readline",
  #          [](BufferedInputStream* self) {
  #            py::gil_scoped_release release;
  #            auto output = self->ReadLineAsString();
  #            py::gil_scoped_acquire acquire;
  #            return py::bytes(output);
  #          })

  #     .def("seek",
  #          [](BufferedInputStream* self, tensorflow::int64 pos) {
  #            py::gil_scoped_release release;
  #            tensorflow::MaybeRaiseRegisteredFromStatusWithGIL(self->Seek(pos));
  #          })
  def seek(self, pos: int):
    MaybeRaiseRegisteredFromStatusWithGIL(self.me_.seek(pos))

  #     .def("tell", [](BufferedInputStream* self) {
  #       py::gil_scoped_release release;
  #       return self->Tell();
  def tell(self):
    return self.me_.tell()
#       });
# }
# }  // namespace



'Generated protocol buffer code.'
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
from ....tensorflow.core.framework import tensor_shape_pb2 as tensorflow_dot_core_dot_framework_dot_tensor__shape__pb2
from ....tensorflow.core.framework import types_pb2 as tensorflow_dot_core_dot_framework_dot_types__pb2
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/core/protobuf/remote_tensor_handle.proto', package='tensorflow.eager', syntax='proto3', serialized_options=b'\n\x18org.tensorflow.frameworkB\x18RemoteTensorHandleProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01', create_key=_descriptor._internal_create_key, serialized_pb=b'\n3tensorflow/core/protobuf/remote_tensor_handle.proto\x12\x10tensorflow.eager\x1a,tensorflow/core/framework/tensor_shape.proto\x1a%tensorflow/core/framework/types.proto"i\n\x15ResourceDtypeAndShape\x12#\n\x05dtype\x18\x01 \x01(\x0e2\x14.tensorflow.DataType\x12+\n\x05shape\x18\x02 \x01(\x0b2\x1c.tensorflow.TensorShapeProto"\xcc\x01\n\x12RemoteTensorHandle\x12\r\n\x05op_id\x18\x01 \x01(\x03\x12\x12\n\noutput_num\x18\x02 \x01(\x05\x12\x0e\n\x06device\x18\x03 \x01(\t\x12\x11\n\top_device\x18\x04 \x01(\t\x12#\n\x05dtype\x18\x05 \x01(\x0e2\x14.tensorflow.DataType\x12K\n\x1aresource_dtypes_and_shapes\x18\x06 \x03(\x0b2\'.tensorflow.eager.ResourceDtypeAndShapeB\x90\x01\n\x18org.tensorflow.frameworkB\x18RemoteTensorHandleProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01b\x06proto3', dependencies=[tensorflow_dot_core_dot_framework_dot_tensor__shape__pb2.DESCRIPTOR, tensorflow_dot_core_dot_framework_dot_types__pb2.DESCRIPTOR])
_RESOURCEDTYPEANDSHAPE = _descriptor.Descriptor(name='ResourceDtypeAndShape', full_name='tensorflow.eager.ResourceDtypeAndShape', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='dtype', full_name='tensorflow.eager.ResourceDtypeAndShape.dtype', index=0, number=1, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='shape', full_name='tensorflow.eager.ResourceDtypeAndShape.shape', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=158, serialized_end=263)
_REMOTETENSORHANDLE = _descriptor.Descriptor(name='RemoteTensorHandle', full_name='tensorflow.eager.RemoteTensorHandle', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='op_id', full_name='tensorflow.eager.RemoteTensorHandle.op_id', index=0, number=1, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='output_num', full_name='tensorflow.eager.RemoteTensorHandle.output_num', index=1, number=2, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='device', full_name='tensorflow.eager.RemoteTensorHandle.device', index=2, number=3, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='op_device', full_name='tensorflow.eager.RemoteTensorHandle.op_device', index=3, number=4, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='dtype', full_name='tensorflow.eager.RemoteTensorHandle.dtype', index=4, number=5, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='resource_dtypes_and_shapes', full_name='tensorflow.eager.RemoteTensorHandle.resource_dtypes_and_shapes', index=5, number=6, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=266, serialized_end=470)
_RESOURCEDTYPEANDSHAPE.fields_by_name['dtype'].enum_type = tensorflow_dot_core_dot_framework_dot_types__pb2._DATATYPE
_RESOURCEDTYPEANDSHAPE.fields_by_name['shape'].message_type = tensorflow_dot_core_dot_framework_dot_tensor__shape__pb2._TENSORSHAPEPROTO
_REMOTETENSORHANDLE.fields_by_name['dtype'].enum_type = tensorflow_dot_core_dot_framework_dot_types__pb2._DATATYPE
_REMOTETENSORHANDLE.fields_by_name['resource_dtypes_and_shapes'].message_type = _RESOURCEDTYPEANDSHAPE
DESCRIPTOR.message_types_by_name['ResourceDtypeAndShape'] = _RESOURCEDTYPEANDSHAPE
DESCRIPTOR.message_types_by_name['RemoteTensorHandle'] = _REMOTETENSORHANDLE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
ResourceDtypeAndShape = _reflection.GeneratedProtocolMessageType('ResourceDtypeAndShape', (_message.Message,), {'DESCRIPTOR': _RESOURCEDTYPEANDSHAPE, '__module__': 'tensorflow.core.protobuf.remote_tensor_handle_pb2'})
_sym_db.RegisterMessage(ResourceDtypeAndShape)
RemoteTensorHandle = _reflection.GeneratedProtocolMessageType('RemoteTensorHandle', (_message.Message,), {'DESCRIPTOR': _REMOTETENSORHANDLE, '__module__': 'tensorflow.core.protobuf.remote_tensor_handle_pb2'})
_sym_db.RegisterMessage(RemoteTensorHandle)
DESCRIPTOR._options = None




'Generated protocol buffer code.'
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
from ....tensorflow.core.protobuf import cluster_pb2 as tensorflow_dot_core_dot_protobuf_dot_cluster__pb2
from ....tensorflow.core.protobuf import config_pb2 as tensorflow_dot_core_dot_protobuf_dot_config__pb2
from ....tensorflow.core.protobuf import device_filters_pb2 as tensorflow_dot_core_dot_protobuf_dot_device__filters__pb2
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/core/protobuf/tensorflow_server.proto', package='tensorflow', syntax='proto3', serialized_options=b'\n\x1aorg.tensorflow.distruntimeB\x0cServerProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01', create_key=_descriptor._internal_create_key, serialized_pb=b'\n0tensorflow/core/protobuf/tensorflow_server.proto\x12\ntensorflow\x1a&tensorflow/core/protobuf/cluster.proto\x1a%tensorflow/core/protobuf/config.proto\x1a-tensorflow/core/protobuf/device_filters.proto"\xf5\x01\n\tServerDef\x12\'\n\x07cluster\x18\x01 \x01(\x0b2\x16.tensorflow.ClusterDef\x12\x10\n\x08job_name\x18\x02 \x01(\t\x12\x12\n\ntask_index\x18\x03 \x01(\x05\x127\n\x16default_session_config\x18\x04 \x01(\x0b2\x17.tensorflow.ConfigProto\x12\x10\n\x08protocol\x18\x05 \x01(\t\x12\x0c\n\x04port\x18\x06 \x01(\x05\x12@\n\x16cluster_device_filters\x18\x07 \x01(\x0b2 .tensorflow.ClusterDeviceFiltersB\x86\x01\n\x1aorg.tensorflow.distruntimeB\x0cServerProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01b\x06proto3', dependencies=[tensorflow_dot_core_dot_protobuf_dot_cluster__pb2.DESCRIPTOR, tensorflow_dot_core_dot_protobuf_dot_config__pb2.DESCRIPTOR, tensorflow_dot_core_dot_protobuf_dot_device__filters__pb2.DESCRIPTOR])
_SERVERDEF = _descriptor.Descriptor(name='ServerDef', full_name='tensorflow.ServerDef', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='cluster', full_name='tensorflow.ServerDef.cluster', index=0, number=1, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='job_name', full_name='tensorflow.ServerDef.job_name', index=1, number=2, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='task_index', full_name='tensorflow.ServerDef.task_index', index=2, number=3, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='default_session_config', full_name='tensorflow.ServerDef.default_session_config', index=3, number=4, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='protocol', full_name='tensorflow.ServerDef.protocol', index=4, number=5, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='port', full_name='tensorflow.ServerDef.port', index=5, number=6, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='cluster_device_filters', full_name='tensorflow.ServerDef.cluster_device_filters', index=6, number=7, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=191, serialized_end=436)
_SERVERDEF.fields_by_name['cluster'].message_type = tensorflow_dot_core_dot_protobuf_dot_cluster__pb2._CLUSTERDEF
_SERVERDEF.fields_by_name['default_session_config'].message_type = tensorflow_dot_core_dot_protobuf_dot_config__pb2._CONFIGPROTO
_SERVERDEF.fields_by_name['cluster_device_filters'].message_type = tensorflow_dot_core_dot_protobuf_dot_device__filters__pb2._CLUSTERDEVICEFILTERS
DESCRIPTOR.message_types_by_name['ServerDef'] = _SERVERDEF
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
ServerDef = _reflection.GeneratedProtocolMessageType('ServerDef', (_message.Message,), {'DESCRIPTOR': _SERVERDEF, '__module__': 'tensorflow.core.protobuf.tensorflow_server_pb2'})
_sym_db.RegisterMessage(ServerDef)
DESCRIPTOR._options = None




'Generated protocol buffer code.'
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/core/profiler/protobuf/tf_data_stats.proto', package='tensorflow.profiler', syntax='proto3', serialized_options=None, create_key=_descriptor._internal_create_key, serialized_pb=b'\n5tensorflow/core/profiler/protobuf/tf_data_stats.proto\x12\x13tensorflow.profiler"\x84\x01\n\x0cIteratorStat\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x15\n\rstart_time_ps\x18\x02 \x01(\x03\x12\x13\n\x0bduration_ps\x18\x03 \x01(\x03\x12\x14\n\x0cself_time_ps\x18\x04 \x01(\x03\x12\x13\n\x0bis_blocking\x18\x05 \x01(\x08\x12\x11\n\tnum_calls\x18\x06 \x01(\x03"\xd6\x01\n\x10IteratorMetadata\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x11\n\tparent_id\x18\x02 \x01(\x03\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x11\n\tlong_name\x18\x06 \x01(\t\x12\x10\n\x08is_async\x18\x04 \x01(\x08\x12A\n\x06params\x18\x05 \x03(\x0b21.tensorflow.profiler.IteratorMetadata.ParamsEntry\x1a-\n\x0bParamsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x028\x01"\x87\x02\n\x11InputPipelineStat\x12\x1e\n\x16bottleneck_iterator_id\x18\x02 \x01(\x03\x12&\n\x1ebottleneck_iterator_latency_ps\x18\x03 \x01(\x03\x12Q\n\x0eiterator_stats\x18\x01 \x03(\x0b29.tensorflow.profiler.InputPipelineStat.IteratorStatsEntry\x1aW\n\x12IteratorStatsEntry\x12\x0b\n\x03key\x18\x01 \x01(\x03\x120\n\x05value\x18\x02 \x01(\x0b2!.tensorflow.profiler.IteratorStat:\x028\x01"\xae\x01\n\x15InputPipelineMetadata\x12\n\n\x02id\x18\x01 \x01(\x03\x12J\n\x04type\x18\x02 \x01(\x0e2<.tensorflow.profiler.InputPipelineMetadata.InputPipelineType\x12\x0c\n\x04name\x18\x04 \x01(\t")\n\x11InputPipelineType\x12\x08\n\x04HOST\x10\x00\x12\n\n\x06DEVICE\x10\x01J\x04\x08\x03\x10\x04"\xe9\x01\n\x12InputPipelineStats\x12<\n\x08metadata\x18\x01 \x01(\x0b2*.tensorflow.profiler.InputPipelineMetadata\x12\x16\n\x0eavg_latency_ps\x18\x03 \x01(\x03\x12\x16\n\x0emin_latency_ps\x18\x04 \x01(\x03\x12\x16\n\x0emax_latency_ps\x18\x05 \x01(\x03\x12\x16\n\x0enum_slow_calls\x18\x06 \x01(\x03\x125\n\x05stats\x18\x02 \x03(\x0b2&.tensorflow.profiler.InputPipelineStat"\xef\x02\n\x0bTfDataStats\x12Q\n\x11iterator_metadata\x18\x02 \x03(\x0b26.tensorflow.profiler.TfDataStats.IteratorMetadataEntry\x12M\n\x0finput_pipelines\x18\x01 \x03(\x0b24.tensorflow.profiler.TfDataStats.InputPipelinesEntry\x1a^\n\x15IteratorMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\x03\x124\n\x05value\x18\x02 \x01(\x0b2%.tensorflow.profiler.IteratorMetadata:\x028\x01\x1a^\n\x13InputPipelinesEntry\x12\x0b\n\x03key\x18\x01 \x01(\x03\x126\n\x05value\x18\x02 \x01(\x0b2\'.tensorflow.profiler.InputPipelineStats:\x028\x01"\xbc\x01\n\x18TfDataBottleneckAnalysis\x12\x0c\n\x04host\x18\x01 \x01(\t\x12\x16\n\x0einput_pipeline\x18\x02 \x01(\t\x12\x16\n\x0emax_latency_ps\x18\x03 \x01(\x03\x12\x15\n\riterator_name\x18\x04 \x01(\t\x12\x1a\n\x12iterator_long_name\x18\x05 \x01(\t\x12\x1b\n\x13iterator_latency_ps\x18\x07 \x01(\x03\x12\x12\n\nsuggestion\x18\x06 \x01(\t"\xb2\x02\n\x13CombinedTfDataStats\x12\x16\n\x0eis_input_bound\x18\x03 \x01(\x08\x12\x0f\n\x07summary\x18\x04 \x01(\t\x12J\n\x13bottleneck_analysis\x18\x01 \x03(\x0b2-.tensorflow.profiler.TfDataBottleneckAnalysis\x12P\n\rtf_data_stats\x18\x02 \x03(\x0b29.tensorflow.profiler.CombinedTfDataStats.TfDataStatsEntry\x1aT\n\x10TfDataStatsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12/\n\x05value\x18\x02 \x01(\x0b2 .tensorflow.profiler.TfDataStats:\x028\x01b\x06proto3')
_INPUTPIPELINEMETADATA_INPUTPIPELINETYPE = _descriptor.EnumDescriptor(name='InputPipelineType', full_name='tensorflow.profiler.InputPipelineMetadata.InputPipelineType', filename=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key, values=[_descriptor.EnumValueDescriptor(name='HOST', index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='DEVICE', index=1, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key)], containing_type=None, serialized_options=None, serialized_start=824, serialized_end=865)
_sym_db.RegisterEnumDescriptor(_INPUTPIPELINEMETADATA_INPUTPIPELINETYPE)
_ITERATORSTAT = _descriptor.Descriptor(name='IteratorStat', full_name='tensorflow.profiler.IteratorStat', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='id', full_name='tensorflow.profiler.IteratorStat.id', index=0, number=1, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='start_time_ps', full_name='tensorflow.profiler.IteratorStat.start_time_ps', index=1, number=2, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='duration_ps', full_name='tensorflow.profiler.IteratorStat.duration_ps', index=2, number=3, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='self_time_ps', full_name='tensorflow.profiler.IteratorStat.self_time_ps', index=3, number=4, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='is_blocking', full_name='tensorflow.profiler.IteratorStat.is_blocking', index=4, number=5, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='num_calls', full_name='tensorflow.profiler.IteratorStat.num_calls', index=5, number=6, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=79, serialized_end=211)
_ITERATORMETADATA_PARAMSENTRY = _descriptor.Descriptor(name='ParamsEntry', full_name='tensorflow.profiler.IteratorMetadata.ParamsEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='tensorflow.profiler.IteratorMetadata.ParamsEntry.key', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='tensorflow.profiler.IteratorMetadata.ParamsEntry.value', index=1, number=2, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=383, serialized_end=428)
_ITERATORMETADATA = _descriptor.Descriptor(name='IteratorMetadata', full_name='tensorflow.profiler.IteratorMetadata', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='id', full_name='tensorflow.profiler.IteratorMetadata.id', index=0, number=1, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='parent_id', full_name='tensorflow.profiler.IteratorMetadata.parent_id', index=1, number=2, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='name', full_name='tensorflow.profiler.IteratorMetadata.name', index=2, number=3, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='long_name', full_name='tensorflow.profiler.IteratorMetadata.long_name', index=3, number=6, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='is_async', full_name='tensorflow.profiler.IteratorMetadata.is_async', index=4, number=4, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='params', full_name='tensorflow.profiler.IteratorMetadata.params', index=5, number=5, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_ITERATORMETADATA_PARAMSENTRY], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=214, serialized_end=428)
_INPUTPIPELINESTAT_ITERATORSTATSENTRY = _descriptor.Descriptor(name='IteratorStatsEntry', full_name='tensorflow.profiler.InputPipelineStat.IteratorStatsEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='tensorflow.profiler.InputPipelineStat.IteratorStatsEntry.key', index=0, number=1, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='tensorflow.profiler.InputPipelineStat.IteratorStatsEntry.value', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=607, serialized_end=694)
_INPUTPIPELINESTAT = _descriptor.Descriptor(name='InputPipelineStat', full_name='tensorflow.profiler.InputPipelineStat', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='bottleneck_iterator_id', full_name='tensorflow.profiler.InputPipelineStat.bottleneck_iterator_id', index=0, number=2, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='bottleneck_iterator_latency_ps', full_name='tensorflow.profiler.InputPipelineStat.bottleneck_iterator_latency_ps', index=1, number=3, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='iterator_stats', full_name='tensorflow.profiler.InputPipelineStat.iterator_stats', index=2, number=1, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_INPUTPIPELINESTAT_ITERATORSTATSENTRY], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=431, serialized_end=694)
_INPUTPIPELINEMETADATA = _descriptor.Descriptor(name='InputPipelineMetadata', full_name='tensorflow.profiler.InputPipelineMetadata', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='id', full_name='tensorflow.profiler.InputPipelineMetadata.id', index=0, number=1, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='type', full_name='tensorflow.profiler.InputPipelineMetadata.type', index=1, number=2, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='name', full_name='tensorflow.profiler.InputPipelineMetadata.name', index=2, number=4, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[_INPUTPIPELINEMETADATA_INPUTPIPELINETYPE], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=697, serialized_end=871)
_INPUTPIPELINESTATS = _descriptor.Descriptor(name='InputPipelineStats', full_name='tensorflow.profiler.InputPipelineStats', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='metadata', full_name='tensorflow.profiler.InputPipelineStats.metadata', index=0, number=1, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='avg_latency_ps', full_name='tensorflow.profiler.InputPipelineStats.avg_latency_ps', index=1, number=3, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='min_latency_ps', full_name='tensorflow.profiler.InputPipelineStats.min_latency_ps', index=2, number=4, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='max_latency_ps', full_name='tensorflow.profiler.InputPipelineStats.max_latency_ps', index=3, number=5, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='num_slow_calls', full_name='tensorflow.profiler.InputPipelineStats.num_slow_calls', index=4, number=6, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='stats', full_name='tensorflow.profiler.InputPipelineStats.stats', index=5, number=2, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=874, serialized_end=1107)
_TFDATASTATS_ITERATORMETADATAENTRY = _descriptor.Descriptor(name='IteratorMetadataEntry', full_name='tensorflow.profiler.TfDataStats.IteratorMetadataEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='tensorflow.profiler.TfDataStats.IteratorMetadataEntry.key', index=0, number=1, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='tensorflow.profiler.TfDataStats.IteratorMetadataEntry.value', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1287, serialized_end=1381)
_TFDATASTATS_INPUTPIPELINESENTRY = _descriptor.Descriptor(name='InputPipelinesEntry', full_name='tensorflow.profiler.TfDataStats.InputPipelinesEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='tensorflow.profiler.TfDataStats.InputPipelinesEntry.key', index=0, number=1, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='tensorflow.profiler.TfDataStats.InputPipelinesEntry.value', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1383, serialized_end=1477)
_TFDATASTATS = _descriptor.Descriptor(name='TfDataStats', full_name='tensorflow.profiler.TfDataStats', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='iterator_metadata', full_name='tensorflow.profiler.TfDataStats.iterator_metadata', index=0, number=2, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='input_pipelines', full_name='tensorflow.profiler.TfDataStats.input_pipelines', index=1, number=1, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_TFDATASTATS_ITERATORMETADATAENTRY, _TFDATASTATS_INPUTPIPELINESENTRY], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1110, serialized_end=1477)
_TFDATABOTTLENECKANALYSIS = _descriptor.Descriptor(name='TfDataBottleneckAnalysis', full_name='tensorflow.profiler.TfDataBottleneckAnalysis', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='host', full_name='tensorflow.profiler.TfDataBottleneckAnalysis.host', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='input_pipeline', full_name='tensorflow.profiler.TfDataBottleneckAnalysis.input_pipeline', index=1, number=2, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='max_latency_ps', full_name='tensorflow.profiler.TfDataBottleneckAnalysis.max_latency_ps', index=2, number=3, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='iterator_name', full_name='tensorflow.profiler.TfDataBottleneckAnalysis.iterator_name', index=3, number=4, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='iterator_long_name', full_name='tensorflow.profiler.TfDataBottleneckAnalysis.iterator_long_name', index=4, number=5, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='iterator_latency_ps', full_name='tensorflow.profiler.TfDataBottleneckAnalysis.iterator_latency_ps', index=5, number=7, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='suggestion', full_name='tensorflow.profiler.TfDataBottleneckAnalysis.suggestion', index=6, number=6, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1480, serialized_end=1668)
_COMBINEDTFDATASTATS_TFDATASTATSENTRY = _descriptor.Descriptor(name='TfDataStatsEntry', full_name='tensorflow.profiler.CombinedTfDataStats.TfDataStatsEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='tensorflow.profiler.CombinedTfDataStats.TfDataStatsEntry.key', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='tensorflow.profiler.CombinedTfDataStats.TfDataStatsEntry.value', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1893, serialized_end=1977)
_COMBINEDTFDATASTATS = _descriptor.Descriptor(name='CombinedTfDataStats', full_name='tensorflow.profiler.CombinedTfDataStats', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='is_input_bound', full_name='tensorflow.profiler.CombinedTfDataStats.is_input_bound', index=0, number=3, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='summary', full_name='tensorflow.profiler.CombinedTfDataStats.summary', index=1, number=4, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='bottleneck_analysis', full_name='tensorflow.profiler.CombinedTfDataStats.bottleneck_analysis', index=2, number=1, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='tf_data_stats', full_name='tensorflow.profiler.CombinedTfDataStats.tf_data_stats', index=3, number=2, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_COMBINEDTFDATASTATS_TFDATASTATSENTRY], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1671, serialized_end=1977)
_ITERATORMETADATA_PARAMSENTRY.containing_type = _ITERATORMETADATA
_ITERATORMETADATA.fields_by_name['params'].message_type = _ITERATORMETADATA_PARAMSENTRY
_INPUTPIPELINESTAT_ITERATORSTATSENTRY.fields_by_name['value'].message_type = _ITERATORSTAT
_INPUTPIPELINESTAT_ITERATORSTATSENTRY.containing_type = _INPUTPIPELINESTAT
_INPUTPIPELINESTAT.fields_by_name['iterator_stats'].message_type = _INPUTPIPELINESTAT_ITERATORSTATSENTRY
_INPUTPIPELINEMETADATA.fields_by_name['type'].enum_type = _INPUTPIPELINEMETADATA_INPUTPIPELINETYPE
_INPUTPIPELINEMETADATA_INPUTPIPELINETYPE.containing_type = _INPUTPIPELINEMETADATA
_INPUTPIPELINESTATS.fields_by_name['metadata'].message_type = _INPUTPIPELINEMETADATA
_INPUTPIPELINESTATS.fields_by_name['stats'].message_type = _INPUTPIPELINESTAT
_TFDATASTATS_ITERATORMETADATAENTRY.fields_by_name['value'].message_type = _ITERATORMETADATA
_TFDATASTATS_ITERATORMETADATAENTRY.containing_type = _TFDATASTATS
_TFDATASTATS_INPUTPIPELINESENTRY.fields_by_name['value'].message_type = _INPUTPIPELINESTATS
_TFDATASTATS_INPUTPIPELINESENTRY.containing_type = _TFDATASTATS
_TFDATASTATS.fields_by_name['iterator_metadata'].message_type = _TFDATASTATS_ITERATORMETADATAENTRY
_TFDATASTATS.fields_by_name['input_pipelines'].message_type = _TFDATASTATS_INPUTPIPELINESENTRY
_COMBINEDTFDATASTATS_TFDATASTATSENTRY.fields_by_name['value'].message_type = _TFDATASTATS
_COMBINEDTFDATASTATS_TFDATASTATSENTRY.containing_type = _COMBINEDTFDATASTATS
_COMBINEDTFDATASTATS.fields_by_name['bottleneck_analysis'].message_type = _TFDATABOTTLENECKANALYSIS
_COMBINEDTFDATASTATS.fields_by_name['tf_data_stats'].message_type = _COMBINEDTFDATASTATS_TFDATASTATSENTRY
DESCRIPTOR.message_types_by_name['IteratorStat'] = _ITERATORSTAT
DESCRIPTOR.message_types_by_name['IteratorMetadata'] = _ITERATORMETADATA
DESCRIPTOR.message_types_by_name['InputPipelineStat'] = _INPUTPIPELINESTAT
DESCRIPTOR.message_types_by_name['InputPipelineMetadata'] = _INPUTPIPELINEMETADATA
DESCRIPTOR.message_types_by_name['InputPipelineStats'] = _INPUTPIPELINESTATS
DESCRIPTOR.message_types_by_name['TfDataStats'] = _TFDATASTATS
DESCRIPTOR.message_types_by_name['TfDataBottleneckAnalysis'] = _TFDATABOTTLENECKANALYSIS
DESCRIPTOR.message_types_by_name['CombinedTfDataStats'] = _COMBINEDTFDATASTATS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
IteratorStat = _reflection.GeneratedProtocolMessageType('IteratorStat', (_message.Message,), {'DESCRIPTOR': _ITERATORSTAT, '__module__': 'tensorflow.core.profiler.protobuf.tf_data_stats_pb2'})
_sym_db.RegisterMessage(IteratorStat)
IteratorMetadata = _reflection.GeneratedProtocolMessageType('IteratorMetadata', (_message.Message,), {'ParamsEntry': _reflection.GeneratedProtocolMessageType('ParamsEntry', (_message.Message,), {'DESCRIPTOR': _ITERATORMETADATA_PARAMSENTRY, '__module__': 'tensorflow.core.profiler.protobuf.tf_data_stats_pb2'}), 'DESCRIPTOR': _ITERATORMETADATA, '__module__': 'tensorflow.core.profiler.protobuf.tf_data_stats_pb2'})
_sym_db.RegisterMessage(IteratorMetadata)
_sym_db.RegisterMessage(IteratorMetadata.ParamsEntry)
InputPipelineStat = _reflection.GeneratedProtocolMessageType('InputPipelineStat', (_message.Message,), {'IteratorStatsEntry': _reflection.GeneratedProtocolMessageType('IteratorStatsEntry', (_message.Message,), {'DESCRIPTOR': _INPUTPIPELINESTAT_ITERATORSTATSENTRY, '__module__': 'tensorflow.core.profiler.protobuf.tf_data_stats_pb2'}), 'DESCRIPTOR': _INPUTPIPELINESTAT, '__module__': 'tensorflow.core.profiler.protobuf.tf_data_stats_pb2'})
_sym_db.RegisterMessage(InputPipelineStat)
_sym_db.RegisterMessage(InputPipelineStat.IteratorStatsEntry)
InputPipelineMetadata = _reflection.GeneratedProtocolMessageType('InputPipelineMetadata', (_message.Message,), {'DESCRIPTOR': _INPUTPIPELINEMETADATA, '__module__': 'tensorflow.core.profiler.protobuf.tf_data_stats_pb2'})
_sym_db.RegisterMessage(InputPipelineMetadata)
InputPipelineStats = _reflection.GeneratedProtocolMessageType('InputPipelineStats', (_message.Message,), {'DESCRIPTOR': _INPUTPIPELINESTATS, '__module__': 'tensorflow.core.profiler.protobuf.tf_data_stats_pb2'})
_sym_db.RegisterMessage(InputPipelineStats)
TfDataStats = _reflection.GeneratedProtocolMessageType('TfDataStats', (_message.Message,), {'IteratorMetadataEntry': _reflection.GeneratedProtocolMessageType('IteratorMetadataEntry', (_message.Message,), {'DESCRIPTOR': _TFDATASTATS_ITERATORMETADATAENTRY, '__module__': 'tensorflow.core.profiler.protobuf.tf_data_stats_pb2'}), 'InputPipelinesEntry': _reflection.GeneratedProtocolMessageType('InputPipelinesEntry', (_message.Message,), {'DESCRIPTOR': _TFDATASTATS_INPUTPIPELINESENTRY, '__module__': 'tensorflow.core.profiler.protobuf.tf_data_stats_pb2'}), 'DESCRIPTOR': _TFDATASTATS, '__module__': 'tensorflow.core.profiler.protobuf.tf_data_stats_pb2'})
_sym_db.RegisterMessage(TfDataStats)
_sym_db.RegisterMessage(TfDataStats.IteratorMetadataEntry)
_sym_db.RegisterMessage(TfDataStats.InputPipelinesEntry)
TfDataBottleneckAnalysis = _reflection.GeneratedProtocolMessageType('TfDataBottleneckAnalysis', (_message.Message,), {'DESCRIPTOR': _TFDATABOTTLENECKANALYSIS, '__module__': 'tensorflow.core.profiler.protobuf.tf_data_stats_pb2'})
_sym_db.RegisterMessage(TfDataBottleneckAnalysis)
CombinedTfDataStats = _reflection.GeneratedProtocolMessageType('CombinedTfDataStats', (_message.Message,), {'TfDataStatsEntry': _reflection.GeneratedProtocolMessageType('TfDataStatsEntry', (_message.Message,), {'DESCRIPTOR': _COMBINEDTFDATASTATS_TFDATASTATSENTRY, '__module__': 'tensorflow.core.profiler.protobuf.tf_data_stats_pb2'}), 'DESCRIPTOR': _COMBINEDTFDATASTATS, '__module__': 'tensorflow.core.profiler.protobuf.tf_data_stats_pb2'})
_sym_db.RegisterMessage(CombinedTfDataStats)
_sym_db.RegisterMessage(CombinedTfDataStats.TfDataStatsEntry)
_ITERATORMETADATA_PARAMSENTRY._options = None
_INPUTPIPELINESTAT_ITERATORSTATSENTRY._options = None
_TFDATASTATS_ITERATORMETADATAENTRY._options = None
_TFDATASTATS_INPUTPIPELINESENTRY._options = None
_COMBINEDTFDATASTATS_TFDATASTATSENTRY._options = None




'Generated protocol buffer code.'
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/core/protobuf/transport_options.proto', package='tensorflow', syntax='proto3', serialized_options=b'ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto', create_key=_descriptor._internal_create_key, serialized_pb=b'\n0tensorflow/core/protobuf/transport_options.proto\x12\ntensorflow"*\n\x10RecvBufRespExtra\x12\x16\n\x0etensor_content\x18\x01 \x03(\x0cBWZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_protob\x06proto3')
_RECVBUFRESPEXTRA = _descriptor.Descriptor(name='RecvBufRespExtra', full_name='tensorflow.RecvBufRespExtra', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='tensor_content', full_name='tensorflow.RecvBufRespExtra.tensor_content', index=0, number=1, type=12, cpp_type=9, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=64, serialized_end=106)
DESCRIPTOR.message_types_by_name['RecvBufRespExtra'] = _RECVBUFRESPEXTRA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
RecvBufRespExtra = _reflection.GeneratedProtocolMessageType('RecvBufRespExtra', (_message.Message,), {'DESCRIPTOR': _RECVBUFRESPEXTRA, '__module__': 'tensorflow.core.protobuf.transport_options_pb2'})
_sym_db.RegisterMessage(RecvBufRespExtra)
DESCRIPTOR._options = None




'Generated protocol buffer code.'
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/core/protobuf/verifier_config.proto', package='tensorflow', syntax='proto3', serialized_options=b'\n\x18org.tensorflow.frameworkB\x14VerifierConfigProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01', create_key=_descriptor._internal_create_key, serialized_pb=b'\n.tensorflow/core/protobuf/verifier_config.proto\x12\ntensorflow"\x9b\x01\n\x0eVerifierConfig\x12"\n\x1averification_timeout_in_ms\x18\x01 \x01(\x03\x12=\n\x12structure_verifier\x18\x02 \x01(\x0e2!.tensorflow.VerifierConfig.Toggle"&\n\x06Toggle\x12\x0b\n\x07DEFAULT\x10\x00\x12\x06\n\x02ON\x10\x01\x12\x07\n\x03OFF\x10\x02B\x8c\x01\n\x18org.tensorflow.frameworkB\x14VerifierConfigProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01b\x06proto3')
_VERIFIERCONFIG_TOGGLE = _descriptor.EnumDescriptor(name='Toggle', full_name='tensorflow.VerifierConfig.Toggle', filename=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key, values=[_descriptor.EnumValueDescriptor(name='DEFAULT', index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='ON', index=1, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='OFF', index=2, number=2, serialized_options=None, type=None, create_key=_descriptor._internal_create_key)], containing_type=None, serialized_options=None, serialized_start=180, serialized_end=218)
_sym_db.RegisterEnumDescriptor(_VERIFIERCONFIG_TOGGLE)
_VERIFIERCONFIG = _descriptor.Descriptor(name='VerifierConfig', full_name='tensorflow.VerifierConfig', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='verification_timeout_in_ms', full_name='tensorflow.VerifierConfig.verification_timeout_in_ms', index=0, number=1, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='structure_verifier', full_name='tensorflow.VerifierConfig.structure_verifier', index=1, number=2, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[_VERIFIERCONFIG_TOGGLE], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=63, serialized_end=218)
_VERIFIERCONFIG.fields_by_name['structure_verifier'].enum_type = _VERIFIERCONFIG_TOGGLE
_VERIFIERCONFIG_TOGGLE.containing_type = _VERIFIERCONFIG
DESCRIPTOR.message_types_by_name['VerifierConfig'] = _VERIFIERCONFIG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
VerifierConfig = _reflection.GeneratedProtocolMessageType('VerifierConfig', (_message.Message,), {'DESCRIPTOR': _VERIFIERCONFIG, '__module__': 'tensorflow.core.protobuf.verifier_config_pb2'})
_sym_db.RegisterMessage(VerifierConfig)
DESCRIPTOR._options = None



from __future__ import annotations
from abc import ABC
from typing import Callable, List, Optional, Dict, NamedTuple, Tuple, Union
from functools import wraps as _wraps
import threading as _threading
import os as _os

from . import file_system
from . import file_statistics
from . import errors
from . import misc
from . import core
from . import io
from . import env_time
from . import port
from . import strings
from . import platform
from . import posix_file_system

_name_mutex = _threading.RLock()

class ThreadOptions(NamedTuple):
  """Options to configure a Thread.

  Note that the options are all hints, and the
  underlying implementation may choose to ignore it."""

  stack_size: int = 0  # Thread stack size to use (in bytes).
  guard_size: int = 0  # Guard area size to use near thread stacks to use (in bytes)
  numa_node: int = port.kNUMANoAffinity

class Thread:
  """Represents a thread used to run a TensorFlow function."""

  def __init__(self, thread: _threading.Thread):
    self._thread = thread

  def release(self):
    """Blocks until the thread of control stops running."""
    if self._thread is not None:
      self._thread.join()
      self._thread = None

  def __del__(self):
    self.release()

  def __enter__(self):
    return self

  def __exit__(self, *args):
    self.release()


class Env(ABC):
  """An interface used by the tensorflow implementation to
  access operating system functionality like the filesystem etc.

  Callers may wish to provide a custom Env object to get fine grain
  control.

  All Env implementations are safe for concurrent access from
  multiple threads without any external synchronization."""
  def __init__(self):
    self._file_system_registry = FileSystemRegistryImpl()

  @classmethod
  def default(cls) -> Env:
    """Returns a default environment suitable for the current operating
    system.

    Sophisticated users may wish to provide their own Env
    implementation instead of relying on this default environment.

    The result of Default() belongs to this library and must never be deleted."""
    return misc.putattr(cls, '_default_env', WindowsEnv if platform.is_windows_platform() else PosixEnv)

  def get_file_system_for_file(self, fname: bytes) -> Tuple[errors.Status, Optional[file_system.FileSystem]]:
    """Returns the FileSystem object to handle operations on the file
    specified by 'fname'. The FileSystem object is used as the implementa
    for the file system related (non-virtual) functions that follow.
    Returned FileSystem object is still owned by the Env object and will
    (might) be destroyed when the environment is destroyed."""
    scheme, host, path = io.parse_uri(fname)
    file_system = self._file_system_registry.lookup(scheme.bytes())
    if file_system is None:
      if scheme.empty():
        scheme = core.string_view(b"[local]")
      return errors.Unimplemented("File system scheme '", scheme,
                                  "' not implemented (file: '", fname, "')"), None
    return errors.Status.OK(), file_system

  def stat(self, fname: bytes) -> Tuple[errors.Status, Optional[file_statistics.FileStatistics]]:
    """Obtains statistics for the given path."""
    err, fs = self.get_file_system_for_file(fname)
    if not err.ok():
      return err, None
    stats = file_statistics.FileStatistics()
    err = fs.stat(fname, stats)
    if not err.ok():
      return err, None
    return err, stats

  def has_atomic_move(self, path: bytes) -> Tuple[errors.Status, bool]:
    """Returns whether the given path is on a file system
    that has atomic move capabilities. This can be used
    to determine if there needs to be a temp location to safely write objects.
    The second boolean argument has_atomic_move contains this information.

    Returns one of the following status codes (not guaranteed exhaustive):
     * OK - The path is on a recognized file system,
            so has_atomic_move holds the above information.
     * UNIMPLEMENTED - The file system of the path hasn't been implemented in TF
    """
    err, fs = self.get_file_system_for_file(path)
    if not err.ok():
      return err, False
    return fs.has_atomic_move(path)

  def get_file_size(self, fname) -> Tuple[errors.Status, int]:
    err, fs = self.get_file_system_for_file(fname)
    if not err.ok():
      return err, 0
    return fs.get_file_size(fname)

  def rename_file(self, src, target) -> errors.Status:
    """Renames file src to target. If target already exists, it will be replaced."""
    # FileSystem* src_fs;
    # FileSystem* target_fs;
    # TF_RETURN_IF_ERROR(GetFileSystemForFile(src, &src_fs));
    # TF_RETURN_IF_ERROR(GetFileSystemForFile(target, &target_fs));
    # if (src_fs != target_fs) {
    #   return errors::Unimplemented("Renaming ", src, " to ", target,
    #                                " not implemented");
    # }
    # return src_fs->RenameFile(src, target);
    err, src_fs = self.get_file_system_for_file(src)
    if not err.ok():
      return err
    err, target_fs = self.get_file_system_for_file(target)
    if not err.ok():
      return err
    if src_fs is not target_fs:
      return errors.Unimplemented("Renaming ", src, " to ", target, " not implemented")
    return src_fs.rename_file(src, target)

  def new_random_access_file(self, fname) -> Tuple[errors.Status, Optional[file_system.RandomAccessFile]]:
    """Creates a brand new random access read-only file with the
    specified name.

    On success, returns OK and a pointer to the new file.
    On failure returns non-OK and None. If the file does not exist,
    returns a non-OK status.

    The returned file may be concurrently accessed by multiple threads.

    The ownership of the returned RandomAccessFile is passed to the caller
    and the object should be deleted when is not used. The file object
    shouldn't live longer than the Env object.
    """
    err, fs = self.get_file_system_for_file(fname)
    if not err.ok():
      return err, None
    return fs.new_random_access_file(fname)

  def new_writable_file(self, fname) -> Tuple[errors.Status, Optional[file_system.WritableFile]]:
    """Creates an object that writes to a new file with the specified
    name.

    Deletes any existing file with the same name and creates a
    new file.  On success, reutrns OK and a pointer to the new file
    On failure returns non-OK and None.

    The returned file will only be accessed by one thread at a time.

    The ownership of the returned WritableFile is passed to the caller
    and the object should be deleted when is not used. The file object
    shouldn't live longer than the Env object.
    """
    err, fs = self.get_file_system_for_file(fname)
    if not err.ok():
      return err, None
    return fs.new_writable_file(fname)

  def file_exists(self, fname) -> errors.Status:
    """Returns OK if the named path exists and NOT_FOUND otherwise."""
    err, fs = self.get_file_system_for_file(fname)
    if not err.ok():
      return err
    return fs.file_exists(fname)

  def is_directory(self, name) -> errors.Status:
    """Returns whether the given path is a directory or not.

    Typical return codes (not guaranteed exhaustive):
     * OK - The path exists and is a directory.
     * FAILED_PRECONDITION - The path exists and is not a directory.
     * NOT_FOUND - The path entry does not exist.
     * PERMISSION_DENIED - Insufficient permissions.
     * UNIMPLEMENTED - The file factory doesn't support directories.
    """
    err, fs = self.get_file_system_for_file(name)
    if not err.ok():
      return err
    return fs.is_directory(name)

  # virtual Status GetMatchingPaths(const std::string& pattern,
  #                                 std::vector<string>* results);
  def get_matching_paths(self, pattern) -> Tuple[errors.Status, Optional[List]]:
    """Given a pattern, stores in *results the set of paths that matches
    that pattern. *results is cleared.

    More details about `pattern` in FileSystem::GetMatchingPaths.
    """
    err, fs = self.get_file_system_for_file(pattern)
    if not err.ok():
      return err, None
    return fs.get_matching_paths(pattern)


  def delete_file(self, fname) -> errors.Status:
    """Deletes the named file."""
    err, fs = self.get_file_system_for_file(fname)
    if not err.ok():
      return err
    return fs.delete_file(fname)

  def get_registered_file_system_schemes(self, schemes: List[bytes]) -> errors.Status:
    """Returns the file system schemes registered for this Env."""
    return self._file_system_registry.get_registered_file_system_schemes(schemes)

  def register_file_system_factory(self, scheme: str, factory: file_system.FileSystemRegistry.Factory):
    """Register a file system for a scheme."""
    return self._file_system_registry.register_factory(scheme, factory)

  def register_file_system(self, scheme: str, file_system: file_system.FileSystem):
    """Register a modular file system for a scheme.

    Same as `register_file_system_factory` but for filesystems provided by plugins.

    TODO(mihaimaruseac): After all filesystems are converted, make this be the
    canonical registration function."""
    return self._file_system_registry.register_filesystem(scheme, file_system)

  def now_nanos(self) -> int:
    """Returns the number of nano-seconds since the Unix epoch."""
    return env_time.EnvTime.now_nanos()

  def now_micros(self) -> int:
    """Returns the number of micro-seconds since the Unix epoch."""
    return env_time.EnvTime.now_micros()

  def now_seconds(self) -> int:
    """Returns the number of seconds since the Unix epoch."""
    return env_time.EnvTime.now_seconds()
  
  def sleep_for_microseconds(self, micros: int):
    """Sleeps/delays the thread for the prescribed number of micro-seconds."""
    return port.usleep(micros)
  
  def get_process_id(self) -> int:
    """Returns the process ID of the calling process."""
    return _os.getpid()
  
  def start_thread(self, thread_options: ThreadOptions, name: str, fn: Callable[[], None]) -> Thread:
    """Returns a new thread that is running fn() and is identified
    (for debugging/performance-analysis) by "name".

    Caller takes ownership of the result and must delete it eventually
    (the deletion will block until fn() stops running)."""
    return PThread(thread_options, name, fn)

  def get_current_thread_id(self) -> int:
    """Returns the thread id of calling thread.
    Posix: Returns pthread id which is only guaranteed to be unique within a
           process.
    Windows: Returns thread id which is unique."""
    return port.get_current_thread_id()

  def get_current_thread_name(self) -> Optional[str]:
    """Returns the current thread name if possible, otherwise None."""
    with _name_mutex:
      if port.get_current_thread_id() in get_thread_name_registry():
        name = get_thread_name_registry()[port.get_current_thread_id()]
        return name
      return None
  
  def sched_closure(self, closure: Callable[[], None]):
    """Schedules the given closure on a thread-pool.

    NOTE(mrry): This closure may block."""
    _threading.Thread(target=closure, daemon=True).start()

  def sched_closure_after(self, micros: int, closure: Callable[[], None]):
    """Schedules the given closure on a thread-pool after the given number
    of microseconds.

    NOTE(mrry): This closure must not block."""
    @_wraps(closure)
    def thunk():
      self.sleep_for_microseconds(micros)
      closure()
    return self.sched_closure(thunk)

class ThreadParams(NamedTuple):
  name: str
  fn: Callable[[], None]

class PThread(Thread):
  def __init__(self, thread_options: ThreadOptions, name: str, fn: Callable[[], None]):
    params = ThreadParams(name, fn)
    thread = _threading.Thread(target=_thread_fn, args=(params,), daemon=True)
    super().__init__(thread)
    thread.start()

def _thread_fn(params: ThreadParams):
  with _name_mutex:
    get_thread_name_registry()[port.get_current_thread_id()] = params.name
  params.fn()
  with _name_mutex:
    del get_thread_name_registry()[port.get_current_thread_id()]

def get_thread_name_registry() -> Dict[int, str]:
  return misc.putattr(get_thread_name_registry, 'singleton', lambda: dict())

class PosixEnv(Env, ABC):
  pass

class WindowsEnv(Env, ABC):
  pass

class FileSystemRegistryImpl(file_system.FileSystemRegistry):
  _registry: Dict[bytes, file_system.FileSystem]

  def __init__(self):
    self._mu = _threading.RLock()
    self._registry = dict()

  def register_factory(self, scheme: bytes, factory: file_system.FileSystemRegistry.Factory) -> errors.Status:
    with self._mu:
      scheme = core.string_view(scheme).bytes()
      if scheme in self._registry:
        return errors.AlreadyExists("File factory for ", scheme,
                                    " already registered")
      self._registry[scheme] = factory()
      return errors.Status.OK()

  def register_filesystem(self, scheme: bytes, filesystem: file_system.FileSystem) -> errors.Status:
    with self._mu:
      scheme = core.string_view(scheme).bytes()
      if scheme in self._registry:
        return errors.AlreadyExists("File factory for ", scheme,
                                    " already registered")
      self._registry[scheme] = filesystem
      return errors.Status.OK()

  def lookup(self, scheme: bytes) -> Optional[file_system.FileSystem]:
    with self._mu:
      scheme = core.string_view(scheme).bytes()
      if scheme in self._registry:
        return self._registry[scheme]

  def get_registered_file_system_schemes(self, schemes: List[bytes]) -> errors.Status:
    with self._mu:
      for e in self._registry.keys():
        schemes.append(e)
      return errors.Status.OK()


Env.default().register_file_system_factory("", posix_file_system.PosixFileSystem)

def getenv(name: str) -> str:
  return _os.getenv(str(name))

def setenv(name: str, value: str, overwrite: int) -> int:
  if overwrite:
    _os.environ[str(name)] = str(value)
  else:
    _os.environ.setdefault(str(name), str(value))
  return 0

def unsetenv(name: str) -> int:
  _os.unsetenv(str(name))
  return 0



import time as _time

class EnvTime:
  """An interface used by the implementation to
  access timer related operations."""
  kMicrosToPicos: int = 1000 * 1000
  kMicrosToNanos: int = 1000
  kMillisToMicros: int = 1000
  kMillisToNanos: int = 1000 * 1000
  kNanosToPicos: int = 1000
  kSecondsToMillis: int = 1000
  kSecondsToMicros: int = 1000 * 1000
  kSecondsToNanos: int = 1000 * 1000 * 1000

  @classmethod
  def now_nanos(cls) -> int:
    """Returns the number of nano-seconds since the Unix epoch."""
    return _time.time_ns()

  @classmethod
  def now_micros(cls) -> int:
    """Returns the number of micro-seconds since the Unix epoch."""
    return cls.now_nanos() // cls.kMicrosToNanos

  @classmethod
  def now_seconds(cls) -> int:
    """Returns the number of seconds since the Unix epoch."""
    return cls.now_nanos() // cls.kSecondsToNanos
